@@ -11,7 +11,7 @@ pub use output_format::*;
 use crate::{
     errors::{FromMessagesError, ParseTestListError, WriteTestListError},
     helpers::write_test_name,
-    target_runner::TargetRunner,
+    target_runner::{PlatformRunner, TargetRunner},
     test_filter::TestFilterBuilder,
 };
 use camino::{Utf8Path, Utf8PathBuf};
@@ -21,7 +21,10 @@ use guppy::{
     graph::{PackageGraph, PackageMetadata},
     PackageId,
 };
-use nextest_metadata::{RustTestCaseSummary, RustTestSuiteSummary, TestListSummary};
+use nextest_metadata::{
+    BinaryListSummary, BuildPlatform, RustTestBinarySummary, RustTestCaseSummary,
+    RustTestSuiteSummary, TestListSummary,
+};
 use once_cell::sync::OnceCell;
 use owo_colors::{OwoColorize, Style};
 use std::{collections::BTreeMap, io, io::Write};
@@ -48,48 +51,62 @@ pub struct RustTestArtifact<'g> {
     /// The working directory that this test should be executed in. If None, the current directory
     /// will not be changed.
     pub cwd: Utf8PathBuf,
+
+    /// The platform for which this test artifact was built.
+    pub build_platform: BuildPlatform,
 }
 
-impl<'g> RustTestArtifact<'g> {
+/// A Rust test binary built by Cargo.
+pub struct RustTestBinary {
+    /// A unique ID.
+    pub id: String,
+    /// The path to the binary artifact.
+    pub path: Utf8PathBuf,
+    /// The package this artifact belongs to.
+    pub package_id: String,
+    /// The unique binary name defined in `Cargo.toml` or inferred by the filename.
+    pub name: String,
+    /// Platform for which this binary was built.
+    /// (Proc-macro tests are built for the host.)
+    pub build_platform: BuildPlatform,
+}
+
+/// The list of Rust test binaries built by Cargo.
+pub struct BinaryList {
+    /// The list of test binaries.
+    pub rust_binaries: Vec<RustTestBinary>,
+}
+
+impl BinaryList {
     /// Parses Cargo messages from the given `BufRead` and returns a list of test binaries.
     pub fn from_messages(
-        graph: &'g PackageGraph,
         reader: impl io::BufRead,
-    ) -> Result<Vec<Self>, FromMessagesError> {
-        let mut binaries = vec![];
+        graph: &PackageGraph,
+    ) -> Result<Self, FromMessagesError> {
+        let mut rust_binaries = vec![];
 
         for message in Message::parse_stream(reader) {
             let message = message.map_err(FromMessagesError::ReadMessages)?;
             match message {
                 Message::CompilerArtifact(artifact) if artifact.profile.test => {
-                    if let Some(binary) = artifact.executable {
+                    if let Some(path) = artifact.executable {
+                        let package_id = artifact.package_id.repr;
+
                         // Look up the executable by package ID.
-                        let package_id = PackageId::new(artifact.package_id.repr);
                         let package = graph
-                            .metadata(&package_id)
+                            .metadata(&PackageId::new(package_id.clone()))
                             .map_err(FromMessagesError::PackageGraph)?;
-                        // Tests are run in the directory containing Cargo.toml
-                        let cwd = package
-                            .manifest_path()
-                            .parent()
-                            .unwrap_or_else(|| {
-                                panic!(
-                                    "manifest path {} doesn't have a parent",
-                                    package.manifest_path()
-                                )
-                            })
-                            .to_path_buf();
 
                         // Construct the binary ID from the package and build target.
-                        let mut binary_id = package.name().to_owned();
-                        let binary_name = artifact.target.name;
+                        let mut id = package.name().to_owned();
+                        let name = artifact.target.name;
 
                         // To ensure unique binary IDs, we use the following scheme:
                         // 1. If the target is a lib, use the package name.
                         //      There can only be one lib per package, so this
                         //      will always be unique.
                         if !artifact.target.kind.contains(&"lib".to_owned()) {
-                            binary_id.push_str("::");
+                            id.push_str("::");
 
                             match artifact.target.kind.get(0) {
                                 // 2. For integration tests, use the target name.
@@ -97,30 +114,38 @@ impl<'g> RustTestArtifact<'g> {
                                 //      kind of targets in a package, so these
                                 //      will always be unique.
                                 Some(kind) if kind == "test" => {
-                                    binary_id.push_str(&binary_name);
+                                    id.push_str(&name);
                                 }
                                 // 3. For all other target kinds, use a
                                 //      combination of the target kind and
                                 //      the target name. For the same reason
                                 //      as above, these will always be unique.
                                 Some(kind) => {
-                                    binary_id.push_str(&format!("{}/{}", kind, binary_name));
+                                    id.push_str(&format!("{}/{}", kind, name));
                                 }
                                 None => {
                                     return Err(FromMessagesError::MissingTargetKind {
                                         package_name: package.name().to_owned(),
-                                        binary_name: binary_name.clone(),
+                                        binary_name: name.clone(),
                                     });
                                 }
                             }
                         }
 
-                        binaries.push(RustTestArtifact {
-                            binary_id,
-                            package,
-                            binary_path: binary,
-                            binary_name,
-                            cwd,
+                        let platform = if artifact.target.kind.len() == 1
+                            && artifact.target.kind.get(0).map(String::as_str) == Some("proc-macro")
+                        {
+                            BuildPlatform::Host
+                        } else {
+                            BuildPlatform::Target
+                        };
+
+                        rust_binaries.push(RustTestBinary {
+                            path,
+                            package_id,
+                            name,
+                            id,
+                            build_platform: platform,
                         })
                     }
                 }
@@ -130,6 +155,142 @@ impl<'g> RustTestArtifact<'g> {
             }
         }
 
+        rust_binaries.sort_by(|b1, b2| b1.id.cmp(&b2.id));
+
+        Ok(Self { rust_binaries })
+    }
+
+    /// Constructs the list from its summary format
+    pub fn from_summary(summary: BinaryListSummary) -> Self {
+        let rust_binaries = summary
+            .rust_binaries
+            .into_values()
+            .map(|bin| RustTestBinary {
+                name: bin.binary_name,
+                path: bin.binary_path,
+                package_id: bin.package_id,
+                id: bin.binary_id,
+                build_platform: bin.build_platform,
+            })
+            .collect();
+        Self { rust_binaries }
+    }
+
+    /// Outputs this list to the given writer.
+    pub fn write(
+        &self,
+        output_format: OutputFormat,
+        writer: impl Write,
+        colorize: bool,
+    ) -> Result<(), WriteTestListError> {
+        match output_format {
+            OutputFormat::Human { verbose } => self
+                .write_human(writer, verbose, colorize)
+                .map_err(WriteTestListError::Io),
+            OutputFormat::Serializable(format) => format
+                .to_writer(&self.to_summary(), writer)
+                .map_err(WriteTestListError::Json),
+        }
+    }
+
+    fn to_summary(&self) -> BinaryListSummary {
+        let rust_binaries = self
+            .rust_binaries
+            .iter()
+            .map(|bin| {
+                let summary = RustTestBinarySummary {
+                    binary_name: bin.name.clone(),
+                    package_id: bin.package_id.clone(),
+                    binary_path: bin.path.clone(),
+                    binary_id: bin.id.clone(),
+                    build_platform: bin.build_platform,
+                };
+                (bin.id.clone(), summary)
+            })
+            .collect();
+
+        BinaryListSummary { rust_binaries }
+    }
+
+    fn write_human(&self, mut writer: impl Write, verbose: bool, colorize: bool) -> io::Result<()> {
+        let mut styles = Styles::default();
+        if colorize {
+            styles.colorize();
+        }
+        for bin in &self.rust_binaries {
+            if verbose {
+                writeln!(writer, "{}:", bin.id.style(styles.binary_id))?;
+                writeln!(writer, "  {} {}", "bin:".style(styles.field), bin.path)?;
+                writeln!(
+                    writer,
+                    "  {} {}",
+                    "build platform:".style(styles.field),
+                    bin.build_platform,
+                )?;
+            } else {
+                writeln!(writer, "{}", bin.id.style(styles.binary_id))?;
+            }
+        }
+        Ok(())
+    }
+
+    /// Outputs this list as a string with the given format.
+    pub fn to_string(&self, output_format: OutputFormat) -> Result<String, WriteTestListError> {
+        // Ugh this sucks. String really should have an io::Write impl that errors on non-UTF8 text.
+        let mut buf = Vec::with_capacity(1024);
+        self.write(output_format, &mut buf, false)?;
+        Ok(String::from_utf8(buf).expect("buffer is valid UTF-8"))
+    }
+}
+
+impl<'g> RustTestArtifact<'g> {
+    /// Constructs a list of test binaries from the list of built binaries.
+    pub fn from_binary_list(
+        graph: &'g PackageGraph,
+        binary_list: BinaryList,
+        path_mapper: Option<&PathMapper>,
+        platform_filter: Option<BuildPlatform>,
+    ) -> Result<Vec<Self>, FromMessagesError> {
+        let mut binaries = vec![];
+
+        for binary in binary_list.rust_binaries {
+            if platform_filter.is_some() && platform_filter != Some(binary.build_platform) {
+                continue;
+            }
+
+            // Look up the executable by package ID.
+            let package_id = PackageId::new(binary.package_id);
+            let package = graph
+                .metadata(&package_id)
+                .map_err(FromMessagesError::PackageGraph)?;
+
+            // Tests are run in the directory containing Cargo.toml
+            let cwd = package
+                .manifest_path()
+                .parent()
+                .unwrap_or_else(|| {
+                    panic!(
+                        "manifest path {} doesn't have a parent",
+                        package.manifest_path()
+                    )
+                })
+                .to_path_buf();
+
+            let (binary_path, cwd) = match path_mapper {
+                Some(mapper) => (mapper.map_binary(binary.path), mapper.map_cwd(cwd)),
+                None => (binary.path, cwd),
+            };
+
+            binaries.push(RustTestArtifact {
+                binary_id: binary.id,
+                package,
+                binary_path,
+                binary_name: binary.name,
+                cwd,
+                build_platform: binary.build_platform,
+            })
+        }
+
         Ok(binaries)
     }
 }
@@ -139,7 +300,6 @@ impl<'g> RustTestArtifact<'g> {
 pub struct TestList<'g> {
     test_count: usize,
     rust_suites: BTreeMap<Utf8PathBuf, RustTestSuite<'g>>,
-    styles: Box<Styles>,
     // Computed on first access.
     skip_count: OnceCell<usize>,
 }
@@ -162,16 +322,66 @@ pub struct RustTestSuite<'g> {
     /// will not be changed.
     pub cwd: Utf8PathBuf,
 
+    /// The platform the test suite is for (host or target).
+    pub build_platform: BuildPlatform,
+
     /// Test case names and other information about them.
     pub testcases: BTreeMap<String, RustTestCaseSummary>,
 }
 
+/// A helper for path remapping.
+///
+/// This is useful when running tests in a different directory, or a different computer, from building them.
+#[derive(Default)]
+pub struct PathMapper {
+    workspace: Option<(Utf8PathBuf, Utf8PathBuf)>,
+    binaries_dir: Option<Utf8PathBuf>,
+}
+
+impl PathMapper {
+    /// Constructs the path mapper.
+    pub fn new(
+        graph: &PackageGraph,
+        workspace: Option<Utf8PathBuf>,
+        binaries_dir: Option<Utf8PathBuf>,
+    ) -> Option<Self> {
+        if workspace.is_none() && binaries_dir.is_none() {
+            return None;
+        }
+
+        Some(Self {
+            workspace: workspace.map(|w| (graph.workspace().root().to_owned(), w)),
+            binaries_dir,
+        })
+    }
+
+    fn map_cwd(&self, path: Utf8PathBuf) -> Utf8PathBuf {
+        match &self.workspace {
+            Some((from, to)) => match path.strip_prefix(from) {
+                Ok(p) => to.join(p),
+                Err(_) => path,
+            },
+            None => path,
+        }
+    }
+
+    fn map_binary(&self, path: Utf8PathBuf) -> Utf8PathBuf {
+        match &self.binaries_dir {
+            Some(dir) => match path.file_name() {
+                Some(file) => dir.join(file),
+                None => path,
+            },
+            None => path,
+        }
+    }
+}
+
 impl<'g> TestList<'g> {
     /// Creates a new test list by running the given command and applying the specified filter.
     pub fn new(
         test_artifacts: impl IntoIterator<Item = RustTestArtifact<'g>>,
         filter: &TestFilterBuilder,
-        runner: Option<&TargetRunner>,
+        runner: &TargetRunner,
     ) -> Result<Self, ParseTestListError> {
         let mut test_count = 0;
 
@@ -193,7 +403,6 @@ impl<'g> TestList<'g> {
         Ok(Self {
             rust_suites: test_artifacts,
             test_count,
-            styles: Box::new(Styles::default()),
             skip_count: OnceCell::new(),
         })
     }
@@ -224,16 +433,10 @@ impl<'g> TestList<'g> {
         Ok(Self {
             rust_suites: test_artifacts,
             test_count,
-            styles: Box::new(Styles::default()),
             skip_count: OnceCell::new(),
         })
     }
 
-    /// Colorizes output.
-    pub fn colorize(&mut self) {
-        self.styles.colorize();
-    }
-
     /// Returns the total number of tests across all binaries.
     pub fn test_count(&self) -> usize {
         self.test_count
@@ -271,11 +474,20 @@ impl<'g> TestList<'g> {
             .rust_suites
             .iter()
             .map(|(binary_path, info)| {
+                let platform = if info.package.is_proc_macro() {
+                    BuildPlatform::Host
+                } else {
+                    BuildPlatform::Target
+                };
                 let testsuite = RustTestSuiteSummary {
                     package_name: info.package.name().to_owned(),
-                    binary_name: info.binary_name.clone(),
-                    package_id: info.package.id().repr().to_owned(),
-                    binary_path: binary_path.clone(),
+                    binary: RustTestBinarySummary {
+                        binary_name: info.binary_name.clone(),
+                        package_id: info.package.id().repr().to_owned(),
+                        binary_path: binary_path.clone(),
+                        binary_id: info.binary_id.clone(),
+                        build_platform: platform,
+                    },
                     cwd: info.cwd.clone(),
                     testcases: info.testcases.clone(),
                 };
@@ -293,10 +505,11 @@ impl<'g> TestList<'g> {
         &self,
         output_format: OutputFormat,
         writer: impl Write,
+        colorize: bool,
     ) -> Result<(), WriteTestListError> {
         match output_format {
             OutputFormat::Human { verbose } => self
-                .write_human(writer, verbose)
+                .write_human(writer, verbose, colorize)
                 .map_err(WriteTestListError::Io),
             OutputFormat::Serializable(format) => format
                 .to_writer(&self.to_summary(), writer)
@@ -324,7 +537,7 @@ impl<'g> TestList<'g> {
     pub fn to_string(&self, output_format: OutputFormat) -> Result<String, WriteTestListError> {
         // Ugh this sucks. String really should have an io::Write impl that errors on non-UTF8 text.
         let mut buf = Vec::with_capacity(1024);
-        self.write(output_format, &mut buf)?;
+        self.write(output_format, &mut buf, false)?;
         Ok(String::from_utf8(buf).expect("buffer is valid UTF-8"))
     }
 
@@ -338,7 +551,6 @@ impl<'g> TestList<'g> {
         Self {
             test_count: 0,
             rust_suites: BTreeMap::new(),
-            styles: Box::new(Styles::default()),
             skip_count: OnceCell::new(),
         }
     }
@@ -382,6 +594,7 @@ impl<'g> TestList<'g> {
             binary_path,
             binary_name,
             cwd,
+            build_platform: platform,
         } = test_binary;
 
         Ok((
@@ -392,6 +605,7 @@ impl<'g> TestList<'g> {
                 binary_name,
                 testcases: tests,
                 cwd,
+                build_platform: platform,
             },
         ))
     }
@@ -431,12 +645,23 @@ impl<'g> TestList<'g> {
         })
     }
 
-    fn write_human(&self, mut writer: impl Write, verbose: bool) -> io::Result<()> {
+    fn write_human(&self, mut writer: impl Write, verbose: bool, colorize: bool) -> io::Result<()> {
+        let mut styles = Styles::default();
+        if colorize {
+            styles.colorize();
+        }
+
         for (test_bin, info) in &self.rust_suites {
-            writeln!(writer, "{}:", info.binary_id.style(self.styles.binary_id))?;
+            writeln!(writer, "{}:", info.binary_id.style(styles.binary_id))?;
             if verbose {
-                writeln!(writer, "  {} {}", "bin:".style(self.styles.field), test_bin)?;
-                writeln!(writer, "  {} {}", "cwd:".style(self.styles.field), info.cwd)?;
+                writeln!(writer, "  {} {}", "bin:".style(styles.field), test_bin)?;
+                writeln!(writer, "  {} {}", "cwd:".style(styles.field), info.cwd)?;
+                writeln!(
+                    writer,
+                    "  {} {}",
+                    "build platform:".style(styles.field),
+                    info.build_platform,
+                )?;
             }
 
             let mut indented = indent_write::io::IndentWriter::new("    ", &mut writer);
@@ -445,7 +670,7 @@ impl<'g> TestList<'g> {
                 writeln!(indented, "(no tests)")?;
             } else {
                 for (name, info) in &info.testcases {
-                    write_test_name(name, &self.styles, &mut indented)?;
+                    write_test_name(name, &styles, &mut indented)?;
                     if !info.filter_match.is_match() {
                         write!(indented, " (skipped)")?;
                     }
@@ -459,16 +684,18 @@ impl<'g> TestList<'g> {
 
 impl<'g> RustTestArtifact<'g> {
     /// Run this binary with and without --ignored and get the corresponding outputs.
-    fn exec(&self, runner: Option<&TargetRunner>) -> Result<(String, String), ParseTestListError> {
-        let non_ignored = self.exec_single(false, runner)?;
-        let ignored = self.exec_single(true, runner)?;
+    fn exec(&self, runner: &TargetRunner) -> Result<(String, String), ParseTestListError> {
+        let platform_runner = runner.for_build_platform(self.build_platform);
+
+        let non_ignored = self.exec_single(false, platform_runner)?;
+        let ignored = self.exec_single(true, platform_runner)?;
         Ok((non_ignored, ignored))
     }
 
     fn exec_single(
         &self,
         ignored: bool,
-        runner: Option<&TargetRunner>,
+        runner: Option<&PlatformRunner>,
     ) -> Result<String, ParseTestListError> {
         let mut argv = Vec::new();
 
@@ -534,16 +761,17 @@ impl<'a> TestInstance<'a> {
     }
 
     /// Creates the command expression for this test instance.
-    pub(crate) fn make_expression(&self, target_runner: Option<&TargetRunner>) -> Expression {
+    pub(crate) fn make_expression(&self, target_runner: &TargetRunner) -> Expression {
+        let platform_runner = target_runner.for_build_platform(self.bin_info.build_platform);
         // TODO: non-rust tests
 
         let mut args = Vec::new();
 
-        let program: std::ffi::OsString = match target_runner {
-            Some(tr) => {
-                args.extend(tr.args());
+        let program: std::ffi::OsString = match platform_runner {
+            Some(runner) => {
+                args.extend(runner.args());
                 args.push(self.binary.as_str());
-                tr.binary().into()
+                runner.binary().into()
             }
             None => {
                 use duct::IntoExecutablePath;
@@ -562,6 +790,8 @@ impl<'a> TestInstance<'a> {
             .dir(&self.bin_info.cwd)
             // This environment variable is set to indicate that tests are being run under nextest.
             .env("NEXTEST", "1")
+            // This environment variable is set to indicate that each test is being run in its own process.
+            .env("NEXTEST_EXECUTION_MODE", "process-per-test")
             // These environment variables are set at runtime by cargo test:
             // https://doc.rust-lang.org/cargo/reference/environment-variables.html#environment-variables-cargo-sets-for-crates
             .env(
@@ -636,7 +866,7 @@ mod tests {
     use std::iter;
 
     #[test]
-    fn test_parse() {
+    fn test_parse_test_list() {
         // Lines ending in ': benchmark' (output by the default Rust bencher) should be skipped.
         let non_ignored_output = indoc! {"
             tests::foo::test_bar: test
@@ -659,6 +889,7 @@ mod tests {
             package: package_metadata(),
             binary_name: fake_binary_name.clone(),
             binary_id: fake_binary_id.clone(),
+            build_platform: BuildPlatform::Target,
         };
         let test_list = TestList::new_with_outputs(
             iter::once((test_binary, &non_ignored_output, &ignored_output)),
@@ -688,6 +919,7 @@ mod tests {
                         },
                     },
                     cwd: fake_cwd,
+                    build_platform: BuildPlatform::Target,
                     package: package_metadata(),
                     binary_name: fake_binary_name,
                     binary_id: fake_binary_id,
@@ -707,6 +939,7 @@ mod tests {
             fake-package::fake-binary:
               bin: /fake/binary
               cwd: /fake/cwd
+              build platform: target
                 tests::baz::test_ignored (skipped)
                 tests::baz::test_quux
                 tests::foo::test_bar
@@ -718,9 +951,11 @@ mod tests {
               "rust-suites": {
                 "fake-package::fake-binary": {
                   "package-name": "metadata-helper",
+                  "binary-id": "fake-package::fake-binary",
                   "binary-name": "fake-binary",
                   "package-id": "metadata-helper 0.1.0 (path+file:///Users/fakeuser/local/testcrates/metadata/metadata-helper)",
                   "binary-path": "/fake/binary",
+                  "build-platform": "target",
                   "cwd": "/fake/cwd",
                   "testcases": {
                     "tests::baz::test_ignored": {
@@ -794,4 +1029,80 @@ mod tests {
             .metadata(&PackageId::new(PACKAGE_METADATA_ID))
             .expect("package ID is valid")
     }
+
+    #[test]
+    fn test_parse_binary_list() {
+        let fake_bin_test = RustTestBinary {
+            id: "fake-package::bin/fake-binary".to_owned(),
+            path: "/fake/binary".into(),
+            package_id: "fake-package 0.1.0 (path+file:///Users/fakeuser/project/fake-package)"
+                .to_owned(),
+            name: "fake-binary".to_owned(),
+            build_platform: BuildPlatform::Target,
+        };
+        let fake_macro_test = RustTestBinary {
+            id: "fake-macro::proc-macro/fake-macro".to_owned(),
+            path: "/fake/macro".into(),
+            package_id: "fake-macro 0.1.0 (path+file:///Users/fakeuser/project/fake-macro)"
+                .to_owned(),
+            name: "fake-macro".to_owned(),
+            build_platform: BuildPlatform::Host,
+        };
+
+        let binary_list = BinaryList {
+            rust_binaries: vec![fake_bin_test, fake_macro_test],
+        };
+
+        // Check that the expected outputs are valid.
+        static EXPECTED_HUMAN: &str = indoc! {"
+        fake-package::bin/fake-binary
+        fake-macro::proc-macro/fake-macro
+        "};
+        static EXPECTED_HUMAN_VERBOSE: &str = indoc! {r#"
+        fake-package::bin/fake-binary:
+          bin: /fake/binary
+          build platform: target
+        fake-macro::proc-macro/fake-macro:
+          bin: /fake/macro
+          build platform: host
+        "#};
+        static EXPECTED_JSON_PRETTY: &str = indoc! {r#"
+        {
+          "rust-binaries": {
+            "fake-macro::proc-macro/fake-macro": {
+              "binary-id": "fake-macro::proc-macro/fake-macro",
+              "binary-name": "fake-macro",
+              "package-id": "fake-macro 0.1.0 (path+file:///Users/fakeuser/project/fake-macro)",
+              "binary-path": "/fake/macro",
+              "build-platform": "host"
+            },
+            "fake-package::bin/fake-binary": {
+              "binary-id": "fake-package::bin/fake-binary",
+              "binary-name": "fake-binary",
+              "package-id": "fake-package 0.1.0 (path+file:///Users/fakeuser/project/fake-package)",
+              "binary-path": "/fake/binary",
+              "build-platform": "target"
+            }
+          }
+        }"#};
+
+        assert_eq!(
+            binary_list
+                .to_string(OutputFormat::Human { verbose: false })
+                .expect("human succeeded"),
+            EXPECTED_HUMAN
+        );
+        assert_eq!(
+            binary_list
+                .to_string(OutputFormat::Human { verbose: true })
+                .expect("human succeeded"),
+            EXPECTED_HUMAN_VERBOSE
+        );
+        assert_eq!(
+            binary_list
+                .to_string(OutputFormat::Serializable(SerializableFormat::JsonPretty))
+                .expect("json-pretty succeeded"),
+            EXPECTED_JSON_PRETTY
+        );
+    }
 }
