@@ -10,7 +10,7 @@ use nextest_metadata::{FilterMatch, MismatchReason};
 use nextest_runner::{
     reporter::TestEvent,
     runner::{ExecutionResult, ExecutionStatuses, RunStats, TestRunner},
-    test_list::RustTestArtifact,
+    test_list::{BinaryList, RustTestArtifact},
 };
 use once_cell::sync::Lazy;
 use std::{
@@ -101,10 +101,32 @@ pub(crate) static EXPECTED_TESTS: Lazy<BTreeMap<&'static str, Vec<TestFixture>>>
             "nextest-tests::example/other" => vec![
                 TestFixture { name: "tests::other_example_success", status: FixtureStatus::Pass },
             ],
+            "nextest-derive::proc-macro/nextest-derive" => vec![
+                TestFixture { name: "it_works", status: FixtureStatus::Pass },
+            ],
         }
     },
 );
 
+pub(crate) static EXPECTED_BINARY_LIST: [(&str, &str, bool); 8] = [
+    (
+        "nextest-derive::proc-macro/nextest-derive",
+        "nextest-derive",
+        false,
+    ),
+    ("nextest-tests", "nextest-tests", true),
+    ("nextest-tests::basic", "basic", true),
+    ("nextest-tests::bin/nextest-tests", "nextest-tests", true),
+    ("nextest-tests::bin/other", "other", true),
+    (
+        "nextest-tests::example/nextest-tests",
+        "nextest-tests",
+        true,
+    ),
+    ("nextest-tests::example/other", "other", true),
+    ("nextest-tests::other", "other", true),
+];
+
 pub(crate) fn workspace_root() -> Utf8PathBuf {
     // one level up from the manifest dir -> into fixtures/nextest-tests
     Utf8Path::new(env!("CARGO_MANIFEST_DIR"))
@@ -113,7 +135,7 @@ pub(crate) fn workspace_root() -> Utf8PathBuf {
         .join("fixtures/nextest-tests")
 }
 
-static PACKAGE_GRAPH: Lazy<PackageGraph> = Lazy::new(|| {
+pub(crate) static PACKAGE_GRAPH: Lazy<PackageGraph> = Lazy::new(|| {
     let mut metadata_command = MetadataCommand::new();
     // Construct a package graph with --no-deps since we don't need full dependency
     // information.
@@ -124,10 +146,10 @@ static PACKAGE_GRAPH: Lazy<PackageGraph> = Lazy::new(|| {
         .expect("building package graph failed")
 });
 
-pub(crate) static FIXTURE_TARGETS: Lazy<BTreeMap<String, RustTestArtifact<'static>>> =
-    Lazy::new(init_fixture_targets);
+pub(crate) static FIXTURE_RAW_CARGO_TEST_OUTPUT: Lazy<Vec<u8>> =
+    Lazy::new(init_fixture_raw_cargo_test_output);
 
-fn init_fixture_targets() -> BTreeMap<String, RustTestArtifact<'static>> {
+fn init_fixture_raw_cargo_test_output() -> Vec<u8> {
     // TODO: actually productionize this, probably requires moving x into this repo
     let cmd_name = match env::var("CARGO") {
         Ok(v) => v,
@@ -135,12 +157,11 @@ fn init_fixture_targets() -> BTreeMap<String, RustTestArtifact<'static>> {
         Err(err) => panic!("error obtaining CARGO env var: {}", err),
     };
 
-    let graph = &*PACKAGE_GRAPH;
-
     let expr = cmd!(
         cmd_name,
         "test",
         "--no-run",
+        "--workspace",
         "--message-format",
         "json-render-diagnostics"
     )
@@ -148,8 +169,18 @@ fn init_fixture_targets() -> BTreeMap<String, RustTestArtifact<'static>> {
     .stdout_capture();
 
     let output = expr.run().expect("cargo test --no-run failed");
+    output.stdout
+}
+
+pub(crate) static FIXTURE_TARGETS: Lazy<BTreeMap<String, RustTestArtifact<'static>>> =
+    Lazy::new(init_fixture_targets);
+
+fn init_fixture_targets() -> BTreeMap<String, RustTestArtifact<'static>> {
+    let graph = &*PACKAGE_GRAPH;
+    let binary_list =
+        BinaryList::from_messages(Cursor::new(&*FIXTURE_RAW_CARGO_TEST_OUTPUT), graph).unwrap();
     let test_artifacts =
-        RustTestArtifact::from_messages(graph, Cursor::new(output.stdout)).unwrap();
+        RustTestArtifact::from_binary_list(graph, binary_list, None, None).unwrap();
 
     test_artifacts
         .into_iter()
